@@ -20,6 +20,13 @@
 #![feature(assert_matches)]
 #![feature(box_vec_non_null)]
 #![feature(char_internals)]
+#![feature(const_alloc_error)]
+#![feature(const_cmp)]
+#![feature(const_convert)]
+#![feature(const_destruct)]
+#![feature(const_heap)]
+#![feature(const_option_ops)]
+#![feature(const_try)]
 #![feature(copied_into_inner)]
 #![feature(core_intrinsics)]
 #![feature(exact_size_is_empty)]
